@@ -156,7 +156,7 @@ KDE_NO_EXPORT bool MediaInfo::wget (const String &str) {
     }
 
     URL uri (str);
-    if (mime.isEmpty () && uri.isLocalFile ()) {
+    if (ActorAgent::Data != type && mime.isEmpty () && uri.isLocalFile ()) {
         mime = MimeType::findByURL (uri);
         if (mrl)
             mrl->mimetype = mime;
@@ -164,7 +164,7 @@ KDE_NO_EXPORT bool MediaInfo::wget (const String &str) {
     if (!mime.isEmpty ())
         maybe_playlist = isPlayListMime (mime);
 
-    if (mrl) {
+    if (ActorAgent::Data != type && mrl) {
         if (mime.isEmpty ()) {
             for (Node *p = mrl->parentNode (); p; p = p->parentNode ()) {
                 Mrl *m = p->mrl ();
@@ -361,15 +361,19 @@ KDE_NO_EXPORT void MediaInfo::create () {
 }
 
 KDE_NO_EXPORT void MediaInfo::ready () {
-    create ();
-    node->document()->post (node, new Posting (node, MsgMediaReady));
+    if (ActorAgent::Data != type)
+        create ();
+    if (ActorAgent::Data == type)
+        node->message (MsgMediaReady);
+    else
+        node->document()->post (node, new Posting (node, MsgMediaReady));
 }
 
-KDE_NO_EXPORT void MediaInfo::jobResult (Job *jb) {
+KDE_NO_EXPORT void MediaInfo::jobResult (IOJob *jb) {
     if (jb->error ())
         data.resize (0);
-    job = 0L; // signal KIO::Job::result deletes itself
-    if (mime.isEmpty ()) {
+    job = 0L; // signal KIO::IOJob::result deletes itself
+    if (ActorAgent::Data != type && mime.isEmpty ()) {
         mime = jb->contentType ();
         if (node->mrl ()) {
             node->mrl ()->mimetype = mime;
@@ -379,7 +383,7 @@ KDE_NO_EXPORT void MediaInfo::jobResult (Job *jb) {
     ready ();
 }
 
-KDE_NO_EXPORT void MediaInfo::jobData (Job *jb, ByteArray &ba) {
+KDE_NO_EXPORT void MediaInfo::jobData (IOJob *jb, ByteArray &ba) {
     //debugLog () << "ActorTypeRuntime::jobData " << data.size () << endl;
     if (ba.size ()) {
         int old_size = data.size ();
@@ -407,6 +411,12 @@ KDE_NO_EXPORT void MediaInfo::jobData (Job *jb, ByteArray &ba) {
     }
 }
 
+KDE_NO_EXPORT void MediaInfo::redirected (IOJob *, const String &url) {
+    Mrl *mrl = node ? node->mrl () : NULL;
+    if (mrl)
+        mrl->src = url;
+}
+
 //------------------------%<----------------------------------------------------
 
 AudioVideoActor::AudioVideoActor (ActorAgent *manager, Node *node)
@@ -579,7 +589,7 @@ void ImageActor::pause () {
 }
 
 KDE_NO_EXPORT
-unsigned ImageActor::render (cairo_surface_t *similar, int width, int height) {
+unsigned ImageActor::render (cairo_surface_t *similar, ISize sz) {
     unsigned penalty = 0;
     if (svg_renderer && update_render) {
         g_object_unref (svg_renderer);
@@ -604,18 +614,18 @@ unsigned ImageActor::render (cairo_surface_t *similar, int width, int height) {
         update_render = false;
     }
     if (svg_renderer &&
-            (cached_img->width != width || cached_img->height != height)) {
+           (cached_img->width != sz.width || cached_img->height != sz.height)) {
         RsvgDimensionData dim;
         rsvg_handle_get_dimensions (svg_renderer, &dim);
         cairo_surface_t *surf = cairo_surface_create_similar (similar,
-                CAIRO_CONTENT_COLOR_ALPHA, width, height);
+                CAIRO_CONTENT_COLOR_ALPHA, sz.width, sz.height);
         cairo_t *cr = cairo_create (surf);
-        cairo_scale (cr, 1.0 * width / dim.width, 1.0 * height / dim.height);
+        cairo_scale (cr, 1.0*sz.width / dim.width, 1.0*sz.height / dim.height);
         rsvg_handle_render_cairo (svg_renderer, cr);
         cairo_destroy (cr);
-        cached_img->setImage (surf, width, height);
+        cached_img->setImage (surf, sz.width, sz.height);
 
-        penalty += width * height / (10 * 480);
+        penalty += sz.width * sz.height / (10 * 480);
         //Image *img = new Image (GPixbuf (
         //            rsvg_handle_get_pixbuf (svg_renderer), true));
         //cached_img->setImage (img);
