@@ -112,6 +112,8 @@ Control::~Control () {
 void Control::init () {
     unsigned int ver, rel, req, evb, err;
     glob_t gt;
+    struct stat st;
+
     use_xvideo = !glob ("/usr/lib/libXv.so*", GLOB_ERR|GLOB_NOSORT, 0, &gt);
     globfree (&gt);
     media_server = new OssoMediaServer (this, m_app->osso_context, use_xvideo);
@@ -137,19 +139,32 @@ void Control::init () {
                         "qgn_grid_tasknavigator_web", 26,
                         (GtkIconLookupFlags)0, 0L),
                     26, 26, GDK_INTERP_BILINEAR));
+
+    const String home = getenv ("HOME");
+    if (!home.isEmpty ()) {
+        data_dir = m_config->readEntry (GCONF_KEY_DATA_DIR,
+                String (home + "/.kmplayer"));
+        if (stat ((const char *) data_dir, &st))
+            mkdir ((const char *) data_dir, 0755);
+    }
+
     Playlist *pldoc = new Playlist (m_app, 0L);
     playlists = pldoc;
-    String path = m_config->readEntry
-        (GCONF_KEY_PLAYLIST_FILE, "/home/user/playlist.xml");
-    const char *tmp = (const char *)path;
-    struct stat st;
-    if (stat (tmp, &st))
-        tmp = "/usr/share/applications/kmplayer/playlist.xml";
+    String path = data_dir + "/playlist.xml";
+    const char *tmp = (const char *) path;
+    if (stat (tmp, &st)) {
+        path = m_config->readEntry (GCONF_KEY_PLAYLIST_FILE,
+                "/home/user/playlist.xml");
+        tmp = (const char *) path;
+        if (stat (tmp, &st))
+            tmp = "/usr/share/applications/kmplayer/playlist.xml";
+    }
     pldoc->path = tmp;
+
     m_app->playlists_tree_id = addTree (m_app, pldoc,
             gtk_icon_theme_load_icon(icon_theme,
                 "kmplayer-fav", 26, (GtkIconLookupFlags)0, 0L));
-    GConfClient *client = (GConfClient *)(GConf &) *m_config;
+    GConfClient *client = (GConfClient *) *m_config;
     GSList *plists = gconf_client_all_entries (client, GCONF_KEY_LISTS, NULL );
     for (GSList *elm = plists; elm; elm = elm->next) {
         GConfEntry *entry = (GConfEntry *)elm->data;
@@ -168,10 +183,24 @@ void Control::init () {
         gconf_entry_free (entry);
     }
     g_slist_free (plists);
-    recents = new KMPlayer::Recents (m_app);
-    m_app->recents_tree_id = addTree (m_app, recents->mrl (),
+
+    Recents *rcdoc = new Recents (m_app);
+    recents = rcdoc;
+    path = data_dir + "/recent.xml";
+    tmp = (const char *)path;
+    if (stat (tmp, &st)) {
+        String old = m_config->readEntry (GCONF_KEY_RECENT_FILE,
+                "/home/user/recent.xml");
+        if (!stat ((const char *) old, &st)) {
+            path = old;
+            tmp = (const char *) path;
+        }
+    }
+    rcdoc->path = tmp;
+    m_app->recents_tree_id = addTree (m_app, rcdoc,
             gtk_icon_theme_load_icon(icon_theme,
                 "qgn_list_gene_url_history", 26, (GtkIconLookupFlags)0, 0L));
+
     String sf = m_config->readEntry (GCONF_KEY_SAVE_FOLDER, String());
     if (!sf.isEmpty ())
         m_app->save_folder = g_strdup ((const char *)sf);
@@ -182,17 +211,15 @@ void Control::deinit () {
     setProcess ((Process *) NULL);
 
     Recents *rec = static_cast <Recents *> (recents.ptr());
-    if (rec->resolved &&
+    if (!data_dir.isEmpty () && rec->resolved &&
             rec->load_tree_version != rec->m_tree_version)
-        rec->writeToFile (m_config->readEntry
-                (GCONF_KEY_RECENT_FILE, "/home/user/recent.xml"));
+        rec->writeToFile (data_dir + "/recent.xml");
     rec->dispose ();
 
     Playlist *playlist = static_cast<Playlist *>(playlists.ptr());
-    if (playlist->resolved &&
+    if (!data_dir.isEmpty () && playlist->resolved &&
             playlist->load_tree_version != playlist->m_tree_version)
-        playlist->writeToFile (m_config->readEntry
-                (GCONF_KEY_PLAYLIST_FILE, "/home/user/playlist.xml"));
+        playlist->writeToFile (data_dir + "/playlist.xml");
     if (m_app->save_folder)
         m_config->writeEntry (GCONF_KEY_SAVE_FOLDER, m_app->save_folder);
     playlist->dispose ();
@@ -200,6 +227,11 @@ void Control::deinit () {
     for (NodePtr n = extra_playlists.first(); n; n = n->nextSibling ())
         n->document()->dispose ();
 
+    for (NodeStoreItem *s = generators.first (); s; s = generators.first ()) {
+        s->data->document ()->dispose ();
+        generators.remove (s);
+    }
+
     m_config->writeEntry (GCONF_KEY_VO, mplayer->hasXv () ? 0 : 1);
 
     delete media_server;
@@ -271,6 +303,12 @@ void Control::stop (bool keep_fullscreen) {
         setCurrent (0L);
     }
     VIEWAREA(m_app)->reset ();
+    app_set_playing (m_app, NOT_PLAYING);
+
+    if (current_generator && current_generator->active ()) {
+        current_generator->deactivate ();
+        current_generator = NULL;
+    }
 
     Node *recent = recents->firstChild ();
     Mrl *mrl = m_process->mrl ();
@@ -282,7 +320,7 @@ void Control::stop (bool keep_fullscreen) {
         int pos = m_process->userStopPosition ();
         pos = pos > 50 ? pos - 50 : 0;
         if (mrl->id == id_node_document)
-            mrl->clip_begin = pos;
+            mrl->clip_begin = 10 * pos;
         String nts;
         int ds = pos % 10;
         if (ds > 0)
@@ -590,7 +628,8 @@ bool Control::setCurrent (NodePtr n) {
 }
 
 static char * stateDesc [] = {
-    "init", "deferred", "activated", "began", "finished", "deactivated"
+    "init", "deferred", "activated", "began", "finished",
+    "deactivated", "resetting"
 };
 
 void Control::stateElementChanged (Node* n, Node::State os, Node::State ns) {
@@ -641,7 +680,7 @@ void Control::updateViewAreaSizes () {
     if (!m_process || !VIEWAREA(m_app)->videoWindow ())
         return;
     VIEWAREA(m_app)->resizeEvent();
-    Rect video_rect = VIEWAREA(m_app)->videoGeometry ();
+    IRect video_rect = VIEWAREA(m_app)->videoGeometry ();
     int x = video_rect.x ();
     int y = video_rect.y ();
 #ifdef __ARMEL__
@@ -699,7 +738,8 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
             }
             if (p == downloader &&
                     !((Downloader *) downloader)->saved_file.isEmpty ())
-                addRecent (((Downloader *) downloader)->saved_file);
+                addRecent (((Downloader *) downloader)->saved_file,
+                        p->mrl ()->caption ());
         }
         // else process changed
         if (!playlist_data->edit_mode)
@@ -755,18 +795,22 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
                 actor->ignore_pause = false;
         }
         setLoading (100); // move possible buffer info dialog
-        addRecent (playlist_data->document->mrl()->src);
+        addRecent (playlist_data->document->mrl()->src,
+                playlist_data->document->caption ());
         updatePlayTree (m_app);
     }
 }
 
-void Control::addRecent (const String &url) {
+void Control::addRecent (const String &url, const String &title) {
     if (url == "Playlist://")
         return;
     recents->defer ();
     Node *c = recents->firstChild ();
     if (!c || c->mrl()->src != url) {
-        recents->insertBefore (new Recent (recents, m_app, url), c);
+        Recent *rec = new Recent (recents, m_app, url);
+        recents->insertBefore (rec, c);
+        if (!title.isEmpty ())
+            rec->setCaption (title);
         c = recents->firstChild ()->nextSibling ();
         int count = 1;
         Node *more = NULL;
@@ -1269,6 +1313,64 @@ void save_locationbar (Application *app) {
     }
 }
 
+static void generatorActivate (GtkMenuItem *item, Control *ctrl) {
+    GList *lst = gtk_container_get_children (
+            GTK_CONTAINER (gtk_widget_get_parent (GTK_WIDGET (item))));
+    NodeStoreItem *store = ctrl->generators.first ();
+    for (GList *l = lst; store && l; l = l->next) {
+        if (GTK_MENU_ITEM (l->data) == item) {
+            if (ctrl->current_generator && ctrl->current_generator->active ())
+                ctrl->current_generator->deactivate ();
+            ctrl->current_generator = store->data;
+            store->data->activate ();
+        }
+        store = store->nextSibling ();
+    }
+    g_list_free (lst);
+}
+
+static void getGenerators (GtkWidget *menu, Control *ctrl, const String &dir) {
+    GDir *gdir = g_dir_open ((const char *) dir, 0, NULL);
+    if (gdir) {
+        debugLog() << dir << endl;
+        const gchar *file = g_dir_read_name (gdir);
+        while (file) {
+            Generator *gen = new Generator (ctrl);
+            NodePtr doc = gen;
+            gen->readFromFile (dir + Char ('/') + file);
+            Node *n = gen->firstChild ();
+            debugLog() << file << " " << n->nodeName() << endl;
+            if (n && n->isElementNode ()) {
+                String name = static_cast <Element *> (n)->getAttribute (
+                        StringPool::attr_name);
+                if (name.isEmpty ())
+                    name = file;
+                GtkWidget *item = gtk_menu_item_new_with_label (name);
+                gtk_menu_append (menu, item);
+                debugLog() << name << endl;
+                g_signal_connect (G_OBJECT (item), "activate",
+                                G_CALLBACK (generatorActivate), ctrl);
+                ctrl->generators.append (new NodeStoreItem (doc));
+            } else {
+                gen->dispose ();
+            }
+            file = g_dir_read_name (gdir);
+        }
+        g_dir_close (gdir);
+    }
+}
+
+void generatorMenuActivate (GtkMenuItem *item, Application *app) {
+    Control *ctrl = CONTROL(app);
+    if (ctrl->generators.first ())
+        return;
+
+    GtkWidget *menu = gtk_menu_item_get_submenu (item);
+    getGenerators (menu, ctrl, "/usr/share/applications/kmplayer/generators");
+    getGenerators (menu, ctrl, ctrl->data_dir + "/generators");
+    gtk_widget_show_all (menu);
+    debugLog () << "Generator menu " << menu << endl;
+}
 /*
  * TODO
 static void cb_iap (struct iap_event_t * event, void * arg) {
