@@ -136,7 +136,7 @@ KDE_NO_EXPORT const char * Node::nodeName () const {
 }
 
 void Node::setState (State nstate) {
-    if (state != nstate) {
+    if (state != nstate && (state_init == nstate || state != state_resetting)) {
         State old = state;
         state = nstate;
         if (document ()->notify_listener)
@@ -208,7 +208,8 @@ void Node::finish () {
 void Node::deactivate () {
     //kdDebug () << nodeName () << " Node::deactivate" << endl;
     bool need_finish (unfinished ());
-    setState (state_deactivated);
+    if (state_resetting != state)
+        setState (state_deactivated);
     for (NodePtr e = firstChild (); e; e = e->nextSibling ()) {
         if (e->state > state_init && e->state < state_deactivated)
             e->deactivate ();
@@ -221,8 +222,10 @@ void Node::deactivate () {
 
 void Node::reset () {
     //kdDebug () << nodeName () << " Node::reset" << endl;
-    if (active ())
+    if (active ()) {
+        setState (state_resetting);
         deactivate ();
+    }
     setState (state_init);
     for (NodePtr e = firstChild (); e; e = e->nextSibling ()) {
         if (e->state != state_init)
@@ -250,21 +253,24 @@ void Node::clearChildren () {
     m_first_child = m_last_child = 0L;
 }
 
-void Node::appendChild (Node *c) {
-    document()->m_tree_version++;
+template <>
+void TreeNode<Node>::appendChild (Node *c) {
+    static_cast <Node *> (this)->document()->m_tree_version++;
     ASSERT (!c->parentNode ());
-    TreeNode<Node>::appendChild (c);
+    appendChildImpl (c);
 }
 
-KDE_NO_EXPORT void Node::insertBefore (Node *c, Node *b) {
+template <>
+KDE_NO_EXPORT void TreeNode<Node>::insertBefore (Node *c, Node *b) {
     ASSERT (!c->parentNode ());
-    document()->m_tree_version++;
-    TreeNode<Node>::insertBefore (c, b);
+    static_cast <Node *> (this)->document()->m_tree_version++;
+    insertBeforeImpl (c, b);
 }
 
-void Node::removeChild (NodePtr c) {
-    document()->m_tree_version++;
-    TreeNode <Node>::removeChild (c);
+template <>
+void TreeNode<Node>::removeChild (NodePtr c) {
+    static_cast <Node *> (this)->document()->m_tree_version++;
+    removeChildImpl (c);
 }
 
 KDE_NO_EXPORT void Node::replaceChild (NodePtr _new, NodePtr old) {
@@ -382,20 +388,27 @@ void Node::closed () {
 }
 
 void *Node::message (MessageType msg, void *content) {
-    if (MsgChildFinished == msg) {
-        Posting *post = (Posting *) content;
-        if (unfinished ()) {
-            if (post->source->state == state_finished)
-                post->source->deactivate ();
-            if (post->source && post->source->nextSibling ())
-                post->source->nextSibling ()->activate ();
-            else
-                finish (); // we're done
+    switch (msg) {
+        case MsgQueryReady:
+            return MsgBool (true);
+
+        case MsgChildFinished: {
+            Posting *post = (Posting *) content;
+            if (unfinished ()) {
+                if (post->source->state == state_finished)
+                    post->source->deactivate ();
+                if (post->source && post->source->nextSibling ())
+                    post->source->nextSibling ()->activate ();
+                else
+                    finish (); // we're done
+            }
+            return NULL;
         }
-        return NULL;
+
+        default:
+            if (MsgStartQueryMessage < msg && MsgEndQueryMessage > msg)
+                return NULL;
     }
-    if (MsgStartQueryMessage < msg && MsgEndQueryMessage > msg)
-        return NULL;
     return MsgUnhandled;
 }
 
@@ -754,8 +767,8 @@ unsigned int Mrl::parseTimeString (const String &ts) {
         d += multiply[mpos++] * t.toDouble();
         s = p >= 0 ? s.left (p) : String ();
     }
-    if (d > 0.1)
-        return (unsigned int) (d * 10);
+    if (d > 0.01)
+        return (unsigned int) (d * 100);
     return 0;
 }
 
