@@ -53,8 +53,6 @@ public:
     typedef SharedPtr <T> SharedType;
     typedef WeakPtr <T> WeakType;
 
-    virtual ~Item () {}
-
     SharedType self () const { return m_self; }
 protected:
     Item ();
@@ -137,8 +135,6 @@ template <class T>
 class KMPLAYER_EXPORT ListNodeBase : public Item <T> {
     friend class List<T>;
 public:
-    virtual ~ListNodeBase () {}
-
     T* nextSibling () const { return m_next.ptr (); }
     T* previousSibling () const { return m_prev.ptr (); }
 protected:
@@ -164,11 +160,9 @@ public:
 template <class T>
 class KMPLAYER_EXPORT TreeNode : public ListNodeBase <T> {
 public:
-    virtual ~TreeNode () {}
-
-    virtual void insertBefore (T *c, T *b);
-    virtual void appendChild (T *c);
-    virtual void removeChild (typename Item<T>::SharedType c);
+    void insertBefore (T *c, T *b);
+    void appendChild (T *c);
+    void removeChild (typename Item<T>::SharedType c);
 
     bool hasChildNodes () const { return m_first_child != 0L; }
     T* parentNode () const { return m_parent.ptr (); }
@@ -177,6 +171,9 @@ public:
 
 protected:
     TreeNode () {}
+    void insertBeforeImpl (T *c, T *b);
+    void appendChildImpl (T *c);
+    void removeChildImpl (typename Item<T>::SharedType c);
     typename Item<T>::WeakType m_parent;
     typename Item<T>::SharedType m_first_child;
     typename Item<T>::WeakType m_last_child;
@@ -233,9 +230,11 @@ enum MessageType
     MsgSurfaceAttach,
     MsgStateFreeze,
     MsgStateRewind,
+    MsgChildReady,
     MsgChildFinished,
 
     MsgStartQueryMessage,
+    MsgQueryReady,
     MsgQueryActorAgent,
     MsgQueryRoleTiming,
     MsgQueryRoleDisplay,
@@ -249,6 +248,9 @@ enum MessageType
 
 #define MsgUnhandled ((void *) 357L)
 
+#define MsgBool(x)                                                          \
+    (void*)(long)(x)
+
 #define nodeMessageReceivers(node, msg)                                     \
     (NodeRefList*)(node)->message (MsgQueryReceivers, (void*)(long)(msg))
 
@@ -267,14 +269,20 @@ typedef Item<AttributeList>::SharedType AttributeListPtr;
 ITEM_AS_POINTER(KMPlayer::AttributeList)
 typedef ListNode<NodePtrW> NodeRefItem;      // Node for ref Nodes
 ITEM_AS_POINTER(KMPlayer::NodeRefItem)
-//typedef ListNode<NodePtr> NodeStoreItem;   // list stores Nodes
+typedef ListNode<NodePtr> NodeStoreItem;   // list stores Nodes
+ITEM_AS_POINTER(KMPlayer::NodeStoreItem)
 typedef NodeRefItem::SharedType NodeRefItemPtr;
 typedef NodeRefItem::WeakType NodeRefItemPtrW;
 typedef List<NodeRefItem> NodeRefList;       // ref nodes, eg. event receivers
+typedef List<NodeStoreItem> NodeStoreList;
 typedef Item<NodeRefList>::SharedType NodeRefListPtr;
 typedef Item<NodeRefList>::WeakType NodeRefListPtrW;
 ITEM_AS_POINTER(KMPlayer::NodeRefList)
 
+template <> void TreeNode<Node>::appendChild (Node *c);
+template <> void TreeNode<Node>::insertBefore (Node *c, Node *b);
+template <> void TreeNode<Node>::removeChild (NodePtr c);
+
 /*
  * Weak ref of the receivers list from signaler and the listener node
  */
@@ -320,7 +328,8 @@ class KMPLAYER_EXPORT Node : public TreeNode <Node> {
 public:
     enum State {
         state_init, state_deferred,
-        state_activated, state_began, state_finished, state_deactivated
+        state_activated, state_began, state_finished,
+        state_deactivated, state_resetting
     };
     enum PlayType {
         play_type_none, play_type_unknown, play_type_info,
@@ -423,9 +432,6 @@ public:
     virtual void reset ();
     virtual void clear ();
     void clearChildren ();
-    void appendChild (Node *c);
-    void insertBefore (Node *c, Node *b);
-    void removeChild (NodePtr c);
     void replaceChild (NodePtr _new, NodePtr old);
     /*
      * Get rid of whitespace only text nodes
@@ -582,8 +588,6 @@ public:
  */
 class KMPLAYER_EXPORT PlayListNotify {
 public:
-    virtual ~PlayListNotify () {}
-
     /**
      * Element has activated or deactivated notification
      */
@@ -938,7 +942,7 @@ inline T* List<T>::item (int i) const {
 }
 
 template <class T>
-inline void TreeNode<T>::appendChild (T *c) {
+inline void TreeNode<T>::appendChildImpl (T *c) {
     if (!m_first_child) {
         m_first_child = c;
         m_last_child = c;
@@ -951,7 +955,7 @@ inline void TreeNode<T>::appendChild (T *c) {
 }
 
 template <class T>
-inline void TreeNode<T>::insertBefore (T *c, T *b) {
+inline void TreeNode<T>::insertBeforeImpl (T *c, T *b) {
     if (!b) {
         appendChild (c);
     } else {
@@ -969,7 +973,7 @@ inline void TreeNode<T>::insertBefore (T *c, T *b) {
 }
 
 template <class T>
-inline void TreeNode<T>::removeChild (typename Item<T>::SharedType c) {
+inline void TreeNode<T>::removeChildImpl (typename Item<T>::SharedType c) {
     if (c->m_prev) {
         c->m_prev->m_next = c->m_next;
     } else
