@@ -788,24 +788,28 @@ void cb_playlistPaste (GtkButton * button, Application * app) {
     }
 }
 
-String getUserInput (Application *app, const gchar *title, const gchar *def) {
+String KMPlayer::getUserInput (Application *app,
+        const gchar *title, const gchar *desc, const gchar *def) {
     GtkWidget * dialog= gtk_dialog_new_with_buttons (
             title,
             GTK_WINDOW (app->window), GTK_DIALOG_MODAL,
             GTK_STOCK_OK, GTK_RESPONSE_OK,
             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
-    GtkWidget *group_edit = gtk_entry_new();
+    if (desc) {
+        GtkWidget *lbl = gtk_label_new (desc);
+        gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), lbl);
+    }
+    GtkWidget *edit = gtk_entry_new();
     if (def)
-        gtk_entry_set_text (GTK_ENTRY (group_edit), def);
-    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
-            group_edit);
+        gtk_entry_set_text (GTK_ENTRY (edit), def);
+    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), edit);
     gtk_window_set_default_size(GTK_WINDOW(dialog), 400, 150);
 
     gtk_widget_show_all (dialog);
 
     String val;
     if (GTK_RESPONSE_OK == gtk_dialog_run (GTK_DIALOG (dialog)))
-        val = gtk_entry_get_text (GTK_ENTRY (group_edit));
+        val = gtk_entry_get_text (GTK_ENTRY (edit));
 
     gtk_widget_destroy (dialog);
 
@@ -819,7 +823,7 @@ void cb_playlistPasteInNewGroup (GtkButton * button, Application * app) {
         Node *node = getNode (pld, path);
         if (node && pld->id == app->playlists_tree_id) {
             String grp = getUserInput (app,
-                    "Enter Name of the Group", "New Group");
+                    "New Group", "Enter name of the group", NULL);
             if (!grp.isNull ()) {
                 Node *newnode = new Group (pld->document, app, grp);
                 newnode->appendChild (CONTROL(app)->copyPasteNode
@@ -865,7 +869,7 @@ void cb_playlistTitle (GtkButton *button, Application *app) {
         Node *node = getNode (pld, path);
         if (node && pld->id == app->playlists_tree_id) {
             String title = node->caption ();
-            title = getUserInput (app, "Enter new Title", title);
+            title = getUserInput (app, "New Title", "Enter new title", title);
             if (!title.isNull ()) {
                 static_cast <Element *> (node)->setAttribute
                     (StringPool::attr_title, title);
