@@ -215,33 +215,174 @@ void Process::setUserStop () {
 
 //-----------------------------------------------------------------------------
 
-static void setupMPlayerFunc (gpointer data) {
-    static_cast <MPlayer *> (data)->setState (Process::Playing); //FIXME
-}
-
-static void watchMPlayerFunc (GPid pid, gint status, gpointer data) {
-    //debugLog () << "watchMPlayerFunc " << endl;
+static void watchProcessFunc (GPid pid, gint status, gpointer data) {
+    //debugLog () << " watchProcessFunc " << endl;
     g_spawn_close_pid (pid);
-    static_cast <MPlayer *> (data)->processExited (pid);
+    static_cast <UnixProcess *> (data)->processExited (pid);
 }
 
 static gboolean watchMPlayerStdin (GIOChannel *, GIOCondition, gpointer data) {
-    static_cast <MPlayer *> (data)->writeEvent ();
+    static_cast <UnixProcess *> (data)->writeEvent ();
     return true;
 }
 
-static gboolean watchMPlayerStdout (GIOChannel *src, GIOCondition, gpointer data) {
-    static_cast <MPlayer *> (data)->readEvent (src);
+static void setupProcessFunc (gpointer data) {
+}
+
+static gboolean
+watchProcessStdout (GIOChannel *src, GIOCondition, gpointer data) {
+    if (G_IO_IN)
+        static_cast <UnixProcess *> (data)->readEvent (src);
     return true;
 }
 
+UnixProcess::UnixProcess (UnixProcessListener *l, int flags)
+    : listener (l), io_flags (flags),
+    pin (0L), pout (0L), win (0), wout (0) {
+}
+
+bool UnixProcess::start (const String &pwd, const String &cmdline) {
+    int argc;
+    gchar **argv;
+    GError *err = 0L;
+    int fdin, fdout, fderr;
+    eof = false;
+    debugLog () << cmdline << endl;
+    if (g_shell_parse_argv ((const char *) cmdline, &argc, &argv, &err) &&
+            g_spawn_async_with_pipes ((const char *) pwd,
+                argv,
+                0L,
+                (GSpawnFlags)(G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD),
+                setupProcessFunc,
+                static_cast <void*> (this),
+                &process_id,
+                &fdin,
+                &fdout,
+                0L /*&fderr*/,
+                &err)) {
+        //darn glib: assertion err!=null g_error_free (err);
+        g_strfreev (argv);
+        g_child_watch_add(process_id, watchProcessFunc,static_cast<void*>(this));
+        pin = g_io_channel_unix_new (fdin);
+        pout = g_io_channel_unix_new (fdout);
+        g_io_channel_set_encoding (pout, 0L, 0L);
+        GIOFlags flags = g_io_channel_get_flags (pout);
+        g_io_channel_set_flags (pout, (GIOFlags)(flags|G_IO_FLAG_NONBLOCK), 0L);
+        g_io_channel_set_buffer_size (pout, 80);
+        wout = g_io_add_watch (pout, G_IO_IN, watchProcessStdout, this);
+        return true;
+    }
+    errorLog () << err->message << endl;
+    g_error_free (err);
+    return false;
+}
+
+void UnixProcess::stop () {
+    closeStdIn ();
+    closeStdOut ();
+    if (process_id) {
+        ::kill (process_id, SIGINT);
+        for (int n = 0; process_id && n < 100; n++) {
+            usleep (10000); // 10ms
+            while (g_main_context_iteration (0L, false))
+                if (!process_id)
+                    break;
+        }
+        if (process_id) {
+            errorLog () << "interupt process" << endl;
+            ::kill (process_id, SIGTERM);
+            for (int n = 0; process_id && n < 50; n++) {
+                usleep (10000); // 10ms
+                while (g_main_context_iteration (0L, false))
+                    if (!process_id)
+                        break;
+            }
+        }
+        if (process_id)
+            processExited (process_id); // give up
+    }
+}
+
+bool UnixProcess::running () const {
+    return process_id;
+}
+
+void UnixProcess::pollStdIn (bool b) {
+    if (b) {
+        if (!win)
+            win = g_io_add_watch (pin, G_IO_OUT, watchMPlayerStdin, this);
+    } else if (win) {
+        g_source_remove (win);
+        win = 0;
+    }
+}
+
+int UnixProcess::readStdOut (char *buf, int len) {
+    gsize nr = 0;;
+    if (pout) {
+        GIOStatus status = g_io_channel_read_chars (pout, buf, len, &nr, 0L);
+        if (status == G_IO_STATUS_EOF || status == G_IO_STATUS_ERROR /*FIXME*/)
+            eof = true;
+    } else {
+        eof = true;
+    }
+    return nr;
+}
+
+int UnixProcess::writeStdIn (const char *data, int len) {
+    gsize nr = 0;
+    g_io_channel_write_chars (pin, data, len, &nr, 0L);
+    g_io_channel_flush (pin, 0L);
+    return nr;
+}
+
+void UnixProcess::closeStdIn () {
+    pollStdIn (false);
+    if (pin) {
+        g_io_channel_shutdown (pin, true, 0L);
+        g_io_channel_unref (pin);
+        pin = 0L;
+    }
+}
+
+void UnixProcess::closeStdOut () {
+    if (wout) {
+        g_source_remove (wout);
+        wout = 0;
+    }
+    if (pout) {
+        g_io_channel_shutdown (pout, true, 0L);
+        g_io_channel_unref (pout);
+        pout = 0L;
+    }
+}
+
+void UnixProcess::readEvent (GIOChannel *channel) {
+    listener->readEvent (this);
+}
+
+void UnixProcess::writeEvent () {
+    if (pin)
+        listener->writeEvent (this);
+}
+
+void UnixProcess::processExited (GPid pid) {
+    if (pid == process_id) {
+        process_id = 0;
+        closeStdIn ();
+        listener->processExited (this);
+    }
+}
+
+//-----------------------------------------------------------------------------
+
 #define NO_SEEK 0x7fffffff
 
 MPlayer::MPlayer (ProcessNotify * ctrl, bool xv)
  : Process (ctrl, "mplayer", xv),
-   pin (0L), pout (0L), win (0), wout (0),
+   unix_process (this, UnixProcess::StdIn | UnixProcess::StdOut),
    wait_for_pause (false) {
-    m_rect = Rect (434, 60, 342, 260);
+    m_rect = IRect (434, 60, 342, 260);
 }
 
 MPlayer::~MPlayer () {
@@ -290,8 +431,8 @@ KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
     Mrl *mrl = this->mrl ();
     if (mrl->repeat > 0)
         cmdline += String (" -loop ") + String::number (mrl->repeat+1);
-    if (is_local_file && mrl->clip_begin > 10)
-        cmdline += String (" -ss ") + String::number (mrl->clip_begin/10);
+    if (is_local_file && mrl->clip_begin > 100)
+        cmdline += String (" -ss ") + String::number (mrl->clip_begin/100);
     cmdline += Char (' ');
 
     const TrieString aspect_ratio ("aspectRatio");
@@ -341,38 +482,11 @@ KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
     char *quoted = g_shell_quote ((const char *) m_url);
     cmdline += String (quoted);
     g_free (quoted);
-    debugLog () << cmdline << endl;
-    int argc;
-    gchar **argv;
-    GError *err = 0L;
-    int fdin, fdout, fderr;
-    if (g_shell_parse_argv ((const char *) cmdline, &argc, &argv, &err) &&
-            g_spawn_async_with_pipes ((const char *) wd, 
-                argv,
-                0L,
-                (GSpawnFlags)(G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD),
-                setupMPlayerFunc,
-                static_cast <void*> (this),
-                &process_id,
-                &fdin,
-                &fdout,
-                0L /*&fderr*/,
-                &err)) {
-        //darn glib: assertion err!=null g_error_free (err);
-        g_strfreev (argv);
+    if (unix_process.start (wd, cmdline)) {
         m_mrl = node;
-        g_child_watch_add(process_id,watchMPlayerFunc,static_cast<void*>(this));
-        pin = g_io_channel_unix_new (fdin);
-        pout = g_io_channel_unix_new (fdout);
-        g_io_channel_set_encoding (pout, 0L, 0L);
-        GIOFlags flags = g_io_channel_get_flags (pout);
-        g_io_channel_set_flags (pout, (GIOFlags)(flags|G_IO_FLAG_NONBLOCK), 0L);
-        g_io_channel_set_buffer_size (pout, 80);
-        wout = g_io_add_watch (pout, G_IO_IN, watchMPlayerStdout, this);
+        setState (Process::Buffering); //FIXME
         return true;
     }
-    errorLog () << err->message << endl;
-    g_error_free (err);
     setState (Ready);
     return false;
 }
@@ -404,35 +518,17 @@ KMPLAYER_NO_EXPORT bool MPlayer::quit () {
         request_quit = true;
         stop ();
         while (g_main_context_iteration (0L, false))
-            if (!pin)
+            if (!unix_process.running ())
                 break;
-        for (int n = 0; pin && n < 50; n++) {
+        for (int n = 0; unix_process.running () && n < 50; n++) {
             usleep (10000); // 10ms
             while (g_main_context_iteration (0L, false))
-                if (!pin)
+                if (!unix_process.running ())
                     break;
         }
-        if (pin) {
+        if (unix_process.running ()) {
             errorLog () << "stop MPlayer" << endl;
-            ::kill (process_id, SIGINT);
-            for (int n = 0; pin && n < 100; n++) {
-                usleep (10000); // 10ms
-                while (g_main_context_iteration (0L, false))
-                    if (!pin)
-                        break;
-            }
-            if (pin) {
-                errorLog () << "interupt MPlayer" << endl;
-                ::kill (process_id, SIGTERM);
-                for (int n = 0; pin && n < 50; n++) {
-                    usleep (10000); // 10ms
-                    while (g_main_context_iteration (0L, false))
-                        if (!pin)
-                            break;
-                }
-            }
-            if (pin)
-                processExited (process_id); // give up
+            unix_process.stop ();
         }
         commands.clear ();
         request_quit = false;
@@ -489,7 +585,7 @@ KMPLAYER_NO_EXPORT void MPlayer::setVideoWindow (int x, int y, int w, int h) {
     if (x + w < 0 || y + h < 0 || w <= 0 || h <= 1 /* hack !!*/ ||
                 x > 800 || y > 480 || w > 800 || h > 480)
         return;
-    Rect rect = Rect (x, y, w, h);
+    IRect rect = IRect (x, y, w, h);
     if (rect == m_rect)
         return;
     m_rect = rect;
@@ -501,30 +597,14 @@ KMPLAYER_NO_EXPORT void MPlayer::setVideoWindow (int x, int y, int w, int h) {
     sendCommand (String (buf));
 }
 
-KMPLAYER_NO_EXPORT void MPlayer::processExited (GPid pid) {
-    if (pid == process_id) {
-        process_id = 0;
-        commands.clear ();
-        if (win) {
-            g_source_remove (win);
-            win = 0;
-        }
-        g_io_channel_shutdown (pin, true, 0L);
-        g_io_channel_unref (pin);
-        pin = 0L;
-        g_source_remove (wout);
-        wout = 0;
-        g_io_channel_shutdown (pout, true, 0L);
-        g_io_channel_unref (pout);
-        pout = 0L;
-        if (m_state > Ready && !request_quit)
-            setState (Ready);
-    }
+KMPLAYER_NO_EXPORT void MPlayer::processExited (UnixProcess *) {
+    unix_process.closeStdOut ();
+    commands.clear ();
+    if (m_state > Ready && !request_quit)
+        setState (Ready);
 }
 
-KMPLAYER_NO_EXPORT void MPlayer::writeEvent () {
-    if (!pin)
-        return;
+KMPLAYER_NO_EXPORT void MPlayer::writeEvent (UnixProcess *) {
     String data;
     if (command_chunk.isEmpty ()) {
         if (commands.size ()) {
@@ -535,152 +615,145 @@ KMPLAYER_NO_EXPORT void MPlayer::writeEvent () {
         data = command_chunk;
     debugLog () << "eval " << data /*<< endl*/;
     gsize nr = 0;
-    if (!data.isEmpty ()) {
-        g_io_channel_write_chars(pin, (const char*)data, data.length(), &nr,0L);
-        g_io_channel_flush (pin, 0L);
-    }
+    if (!data.isEmpty ())
+        nr = unix_process.writeStdIn ((const char*) data, data.length ());
     if (nr < data.length ()) {
         //debugLog () << "partial " << nr << " " << data.length() << endl;
         command_chunk = String (((const char*) data) + nr);
     } else if (!commands.size ()) {
         // FIXME: always remove and readd when new data from stdin arrives
-        g_source_remove (win);
-        win = 0;
+        unix_process.pollStdIn (false);
     }
 }
 
-KMPLAYER_NO_EXPORT void MPlayer::readEvent (GIOChannel * src) {
-    gsize nr;
+KMPLAYER_NO_EXPORT void MPlayer::readEvent (UnixProcess *) {
     int inbuf = strlen (outbuf);
-    g_io_channel_read_chars (src, outbuf+inbuf, sizeof (outbuf)-inbuf, &nr, 0L);
+    gsize nr = unix_process.readStdOut (outbuf + inbuf, sizeof (outbuf) -inbuf);
     //debugLog() << "read " << nr << " chars had " << inbuf << endl;
     outbuf[inbuf + nr] = 0;
-    if (src == pout) {
-        char *tok = outbuf;
-        for (int i = 0; i < inbuf + nr; i++) {
-            if (outbuf[i] == '\r' && outbuf[i+1] != '\n') {
-                outbuf[i] = 0;
-                char * c = strstr (tok, "A:");
-                if (!c)
-                    c = strstr (tok, "V:");
+    char *tok = outbuf;
+    for (int i = 0; i < inbuf + nr; i++) {
+        if (outbuf[i] == '\r' && outbuf[i+1] != '\n') {
+            outbuf[i] = 0;
+            char * c = strstr (tok, "A:");
+            if (!c)
+                c = strstr (tok, "V:");
+            if (c) {
+                setPosition ((int)(10 * strtof (c + 2, 0L)));
+                request_seek = NO_SEEK;
+                setState (Playing);
+            } else {
+                c = strstr (tok, "Cache fill:");
                 if (c) {
-                    setPosition ((int)(10 * strtof (c + 2, 0L)));
-                    request_seek = NO_SEEK;
+                    char *p = strchr (tok + 11, '%');
+                    if (p)
+                        *p = 0;
+                    //debugLog() << "c " << (c + 11) << endl;
+                    m_notify->setLoading ((int)strtof (c + 11, 0L));
+                }
+                if (m_state == Paused)
                     setState (Playing);
+            }
+            tok = outbuf + i + 1;
+        } else if (outbuf[i] == '\n') {
+            bool handled = false;
+            outbuf[i] = 0;
+            debugLog() << "> " << tok << endl;
+            if (m_state == Buffering) {
+                char * p = strstr (tok, "Start");
+                if (p && strstr (p, "play")) { // find a 'Start.*play' pattern
+                    m_notify->setLoading (99);
+                    handled = true;
                 } else {
-                    c = strstr (tok, "Cache fill:");
+                    char * c = strstr (tok, "Cache fill:");
                     if (c) {
                         char *p = strchr (tok + 11, '%');
                         if (p)
                             *p = 0;
-                        //debugLog() << "c " << (c + 11) << endl;
+                        //debugLog() << "c2 " << (c + 11) << endl;
                         m_notify->setLoading ((int)strtof (c + 11, 0L));
-                    }
-                    if (m_state == Paused)
-                        setState (Playing);
-                }
-                tok = outbuf + i + 1;
-            } else if (outbuf[i] == '\n') {
-                bool handled = false;
-                outbuf[i] = 0;
-                debugLog() << "> " << tok << endl;
-                if (m_state == Buffering) {
-                    char * p = strstr (tok, "Start");
-                    if (p && strstr (p, "play")) { // find a 'Start.*play' pattern
-                        m_notify->setLoading (99);
                         handled = true;
                     } else {
-                        char * c = strstr (tok, "Cache fill:");
-                        if (c) {
-                            char *p = strchr (tok + 11, '%');
-                            if (p)
-                                *p = 0;
-                            //debugLog() << "c2 " << (c + 11) << endl;
-                            m_notify->setLoading ((int)strtof (c + 11, 0L));
+                        char * v = strstr (tok, "Video: no video");
+                        if (v) {
+                            setHasVideo (false);
                             handled = true;
-                        } else {
-                            char * v = strstr (tok, "Video: no video");
-                            if (v) {
-                                setHasVideo (false);
-                                handled = true;
-                            }
                         }
                     }
                 }
-                if (!handled) {
-                    if (!strncmp (tok, "ID_", 3)) {
-                        handled = true;
-                        if (!strncmp (tok + 3, "LENGTH", 6)) {
-                            m_length = (int)(10 * strtof (tok + 10, 0L));
-                            m_notify->setLength (m_length);
-                        } else if (!strncmp (tok + 3, "PAUSED", 6)) {
-                            if (wait_for_pause) {
-                                wait_for_pause = false;
-                                setState (Paused);
-                            } // else ignore, a second 'pause' cmd should follow
-                        } else if (!strncmp (tok + 3, "VIDEO_WIDTH", 11)) {
-                            m_width = (int) strtof (tok + 15, 0L);
-                            if (m_height > 0)
-                                updateAspects ();
-                        } else if (!strncmp (tok + 3, "VIDEO_HEIGHT", 12)) {
-                            m_height = (int) strtof (tok + 16, 0L);
-                            if (m_width > 0)
-                                updateAspects ();
-                        } else if (!strncmp (tok + 3, "VIDEO_ASPECT", 12)) {
-                            // convert in C locale
-                            m_aspect = (float) String (tok + 16).toDouble ();
-                            if (m_aspect > 0.01)
-                                updateAspects ();
+            }
+            if (!handled) {
+                if (!strncmp (tok, "ID_", 3)) {
+                    handled = true;
+                    if (!strncmp (tok + 3, "LENGTH", 6)) {
+                        m_length = (int)(10 * strtof (tok + 10, 0L));
+                        m_notify->setLength (m_length);
+                    } else if (!strncmp (tok + 3, "PAUSED", 6)) {
+                        if (wait_for_pause) {
+                            wait_for_pause = false;
+                            setState (Paused);
+                        } // else ignore, a second 'pause' cmd should follow
+                    } else if (!strncmp (tok + 3, "VIDEO_WIDTH", 11)) {
+                        m_width = (int) strtof (tok + 15, 0L);
+                        if (m_height > 0)
+                            updateAspects ();
+                    } else if (!strncmp (tok + 3, "VIDEO_HEIGHT", 12)) {
+                        m_height = (int) strtof (tok + 16, 0L);
+                        if (m_width > 0)
+                            updateAspects ();
+                    } else if (!strncmp (tok + 3, "VIDEO_ASPECT", 12)) {
+                        // convert in C locale
+                        m_aspect = (float) String (tok + 16).toDouble ();
+                        if (m_aspect > 0.01)
+                            updateAspects ();
+                    }
+                } else {
+                    char *icy = strstr (tok, "ICY Info");
+                    if (icy) {
+                        //debugLog () << "icy found" << endl;
+                        char *t = strstr (icy + 8, "StreamTitle=");
+                        if (t) {
+                            char *e = strchr (t + 12, ';');
+                            if (e)
+                                *e = 0;
+                            infoMessage (String (t + 12));
                         }
+                        handled = true;
                     } else {
-                        char *icy = strstr (tok, "ICY Info");
-                        if (icy) {
-                            //debugLog () << "icy found" << endl;
-                            char *t = strstr (icy + 8, "StreamTitle=");
-                            if (t) {
-                                char *e = strchr (t + 12, ';');
-                                if (e)
-                                    *e = 0;
-                                infoMessage (String (t + 12));
-                            }
-                            handled = true;
-                        } else {
-                            // Crop: 320 x 200, 0 ; 20
-                            char *crop = strstr (tok, "Crop:");
-                            if (crop) {
-                                char *by = strchr (tok + 5, 'x');
-                                if (by) {
-                                    char *endptr = NULL;
-                                    int w = strtol (tok + 5, &endptr, NULL);
-                                    if (w > 0 && endptr != tok + 5) {
-                                        int h = strtol (by + 1, &endptr, NULL);
-                                        if (h > 0 && endptr != by + 1) {
-                                            m_width = w;
-                                            m_height = h;
-                                            updateAspects ();
-                                            handled = true;
-                                        }
+                        // Crop: 320 x 200, 0 ; 20
+                        char *crop = strstr (tok, "Crop:");
+                        if (crop) {
+                            char *by = strchr (tok + 5, 'x');
+                            if (by) {
+                                char *endptr = NULL;
+                                int w = strtol (tok + 5, &endptr, 10);
+                                if (w > 0 && endptr != tok + 5) {
+                                    int h = strtol (by + 1, &endptr, 10);
+                                    if (h > 0 && endptr != by + 1) {
+                                        m_width = w;
+                                        m_height = h;
+                                        updateAspects ();
+                                        handled = true;
                                     }
                                 }
                             }
                         }
                     }
                 }
-                tok = outbuf + i + 1;
             }
+            tok = outbuf + i + 1;
         }
-        inbuf = tok - outbuf;
-        if (inbuf > 0)
-            memmove (outbuf, tok, inbuf);
-        outbuf[inbuf] = 0;
     }
+    inbuf = tok - outbuf;
+    if (inbuf > 0)
+        memmove (outbuf, tok, inbuf);
+    outbuf[inbuf] = 0;
 }
 
 KMPLAYER_NO_EXPORT bool MPlayer::sendCommand (const String & cmd) {
     if (playing ()) {
         commands.push_back (cmd + Char ('\n'));
-        if (!win)
-            win = g_io_add_watch (pin, G_IO_OUT, watchMPlayerStdin, this);
+        unix_process.pollStdIn (true);
         return true;
     }
     return false;
@@ -1227,7 +1300,7 @@ void OssoMediaServer::setVideoWindow (int x, int y, int w, int h) {
     //debugLog() << "OssoMediaServer::setVideoWindow " << w << "x" << h << endl;
     if (!(x < 0 || y < 0 || w <= 0 || h <= 1 /* hack !!*/ ||
                 x > 800 || y > 480 || w > 800 || h > 480))
-        m_rect = Rect (x, y, w, h);
+        m_rect = IRect (x, y, w, h);
 }
 
 KMPLAYER_NO_EXPORT void OssoMediaServer::processStopped () {
@@ -1263,8 +1336,8 @@ KMPLAYER_NO_EXPORT void OssoMediaServer::processPlaying () {
     if (!pos_timer)
         pos_timer = g_timeout_add (5000, cb_getPosition, this);
     Mrl *m = mrl ();
-    if (is_local_file && m && m->clip_begin > 10)
-        seek (m->clip_begin, true);
+    if (is_local_file && m && m->clip_begin > 100)
+        seek (m->clip_begin/10, true);
 }
 
 KMPLAYER_NO_EXPORT void OssoMediaServer::processPaused () {
@@ -1633,7 +1706,7 @@ KMPLAYER_NO_EXPORT void NpStream::close () {
     job = NULL;
 }
 
-KMPLAYER_NO_EXPORT void NpStream::jobData (Job *jb, ByteArray & data) {
+KMPLAYER_NO_EXPORT void NpStream::jobData (IOJob *jb, ByteArray & data) {
     //debugLog () << "NpStream::jobData " << url.url () << endl;
     if (!bytes)
         nppStreamInfo (stream, jb->contentType (), jb->contentLength ());
@@ -1641,14 +1714,14 @@ KMPLAYER_NO_EXPORT void NpStream::jobData (Job *jb, ByteArray & data) {
     nppStreamData (stream, data.data (), data.size (), NULL /*FIXME*/);
 }
 
-KMPLAYER_NO_EXPORT void NpStream::jobResult (Job *jb) {
+KMPLAYER_NO_EXPORT void NpStream::jobResult (IOJob *jb) {
     debugLog () << "NpStream::jobResult " << bytes << endl;
     finish_reason = jb->error () || !bytes ? BecauseError : BecauseDone;
     job = NULL;
     player->streamFinished (this);
 }
 
-KMPLAYER_NO_EXPORT void NpStream::redirected (Job *, const String & uri) {
+KMPLAYER_NO_EXPORT void NpStream::redirected (IOJob *, const String & uri) {
     url = URL (url, uri);
     nppStreamRedirected (stream, (const char *)url.url ());
 }
@@ -1665,37 +1738,50 @@ Downloader::Downloader (ProcessNotify *ctrl, Application *app)
 Downloader::~Downloader () {
 }
 
+static String genFileName (const String &str, bool skip_on_invalid) {
+    String name;
+    bool valid = true;
+    for (const char *s = (const char *)str; s && *s; ++s)
+        switch (*s) {
+            case '\n':
+            case ';':
+            case ':':
+            case '*':
+            case '?':
+            case ',':
+            case '&':
+            case '>':
+            case '<':
+            case '|':
+            case '"':
+            case '\'':
+                if (!skip_on_invalid)
+                    return String ();
+                break;
+            default:
+                name += Char (*s);
+        }
+    return name;
+}
+
 bool KMPLAYER_NO_EXPORT Downloader::play (NodePtr mrl) {
     Process::play (mrl);
     size = bytes = 0;
     saved_file.clear ();
     if (!m_url.isEmpty ()) {
-        gchar *filename = NULL;
-        String name = mrl->caption ();
-        if (!name.isEmpty ()) {
-            String title = name;
-            name = "";
-            for (const char *s = (const char *)title; s && *s; ++s)
-                switch (*s) {
-                    case '\n':
-                    case ';':
-                    case ':':
-                    case '*':
-                    case '?':
-                    case ',':
-                    case '&':
-                    case '>':
-                    case '<':
-                    case '|':
-                    case '"':
-                    case '\'':
-                        break;
-                    default:
-                        name += Char (*s);
-                }
+        String name;
+        int pos = m_url.lastIndexOf (Char ('/'));
+        if (pos > 0)
+            name = genFileName ((const char *) m_url.mid (pos + 1), false);
+        if (name.isEmpty ()) {
+            name = mrl->caption ();
+            if (!name.isEmpty ())
+                name = genFileName ((const char *) name, true);
         }
-        filename = app_file_chooser (application,
-                (name.isEmpty () ? NULL : (const char *) name),
+        String path = String (application->save_folder);
+        if (!name.isEmpty ())
+            path += name;
+        gchar *filename = app_file_chooser (application, (const char *) path,
                 GTK_FILE_CHOOSER_ACTION_SAVE);
         if (filename) {
             fd = open (filename, O_WRONLY | O_CREAT | O_TRUNC, 0664);
@@ -1703,9 +1789,10 @@ bool KMPLAYER_NO_EXPORT Downloader::play (NodePtr mrl) {
                 job = asyncGet (this, m_url);
                 setState (Buffering);
                 saved_file = filename;
-                g_free (filename);
-                return true;
             }
+            app_store_save_directory (application, filename);
+            if (!saved_file.isEmpty ())
+                return true;
         }
     }
     setState (NotRunning);
@@ -1723,7 +1810,7 @@ KMPLAYER_NO_EXPORT bool Downloader::stop () {
     return true;
 }
 
-KMPLAYER_NO_EXPORT void Downloader::jobData (Job *jb, ByteArray & data) {
+KMPLAYER_NO_EXPORT void Downloader::jobData (IOJob *jb, ByteArray & data) {
     if (!bytes)
         size = jb->contentLength ();
     bytes += data.size ();
@@ -1735,7 +1822,7 @@ KMPLAYER_NO_EXPORT void Downloader::jobData (Job *jb, ByteArray & data) {
         write (fd, data.data (), data.size ());
 }
 
-KMPLAYER_NO_EXPORT void Downloader::jobResult (Job *) {
+KMPLAYER_NO_EXPORT void Downloader::jobResult (IOJob *) {
     m_notify->setLoading (100);
     if (fd > 0)
         close (fd);
@@ -1745,3 +1832,6 @@ KMPLAYER_NO_EXPORT void Downloader::jobResult (Job *) {
         saved_file.clear ();
     setState (NotRunning);
 }
+
+KMPLAYER_NO_EXPORT void Downloader::redirected (IOJob *, const String &) {
+}
