@@ -287,10 +287,18 @@ static NPObject *nsRetainObject (NPObject *npobj) {
     return npobj;
 }
 
-static void nsReleaseObject (NPObject *obj) {
+static void nsReleaseObject (NPObject *npobj) {
+    NPObject *obj = npobj;
     /*print ("NPN_ReleaseObject\n");*/
-    if (! (--obj->referenceCount))
+    if (!obj)
+        obj = scriptable_peer;
+    if (! (--obj->referenceCount)) {
         obj->_class->deallocate (obj);
+        if (obj == scriptable_peer) {
+            print ("NPN_ReleaseObject default\n");
+            scriptable_peer = NULL;
+        }
+    }
 }
 
 static NPError nsGetURL (NPP instance, const char* url, const char* target) {
@@ -504,8 +512,16 @@ static int32_t nsIntFromIdentifier (NPIdentifier identifier) {
 static bool nsInvoke (NPP instance, NPObject * npobj, NPIdentifier method,
         const NPVariant *args, uint32_t arg_count, NPVariant *result) {
     (void)instance;
+    NPObject *obj = npobj;
+    if (!obj) {
+        if (!scriptable_peer &&
+            NPERR_NO_ERROR != np_funcs.getvalue ((void*)npp,
+                NPPVpluginScriptableNPObject, (void*)&scriptable_peer))
+            return NPERR_INVALID_PARAM;
+        obj = scriptable_peer;
+    }
     /*print ("NPN_Invoke %s\n", id);*/
-    return npobj->_class->invoke (npobj, method, args, arg_count, result);
+    return obj->_class->invoke (obj, method, args, arg_count, result);
 }
 
 static bool nsInvokeDefault (NPP instance, NPObject * npobj,
@@ -978,10 +994,6 @@ void *nppInstanceOpen (const char *mime, uint16_t argc, char *argn[],
             nsMemFree (npp);
             return NULL;
         }
-        np_err = np_funcs.getvalue ((void*)npp,
-                NPPVpluginScriptableNPObject, (void*)&scriptable_peer);
-        if (np_err != NPERR_NO_ERROR || !scriptable_peer)
-            print ("NPP_GetValue no NPPVpluginScriptableNPObject %d\n", np_err);
     }
     memset (&np_window, 0, sizeof (NPWindow));
     display = gdk_x11_get_default_xdisplay ();
